/* $Id$ */
// Copyright (C) 2002, International Business Machines
// Corporation and others, Copyright (C) 2012, FasterCoin.  All Rights Reserved.
// This code is licensed under the terms of the Eclipse Public License (EPL).

#include "CoinAbcCommonFactorization.hpp"
#ifndef ABC_JUST_ONE_FACTORIZATION
#define CoinAbcTypeFactorization CoinAbcFactorization
#define ABC_SMALL -1
#include "CoinAbcBaseFactorization.hpp"
#include "CoinAbcBaseFactorization4.cpp"
#endif

/* vi: softtabstop=2 shiftwidth=2 expandtab tabstop=2
*/
